#include <stdio.h>
#include <dos.h>
#include <dir.h>
#include <time.h>
#include <string.h>
#include <bios.h>
#include <sys\stat.h>
#include <stdlib.h>

#define VER "v1.3"
#define RIGHT       0x01
#define LEFT       0x02
#define TRUE 1
#define FALSE 0

void
touch(char[]);
void
main(void);
char           *
upit(char[]);
void
writeout(void);
void
help(void);
void
printlog(void);
void
add2log(void);
void
getlog(char *);

char            filename[255], bday[4], bmon[4], ampm = 'a';
char            set = FALSE, log = FALSE, needhelp = FALSE, cnt, found;
struct ffblk    ffblk;
struct time     curtime;
struct date     curdate;
struct tm       old, *new;
time_t          thetime;
int             sec, min, day, hour, bdate, bhour, bmin, bsec, byear;

void
main(void)
{
   strcpy(filename, searchpath(_argv[0]));
   for (cnt = 1; cnt + 1 <= _argc; cnt++) {
      found = FALSE;
      if (strcmp("SET", upit(_argv[cnt])) == 0)
         set = found = TRUE;
      if (strcmp("LOG", upit(_argv[cnt])) == 0)
         log = found = TRUE;
      if (!found) {
         needhelp = TRUE;
         break;
      }
   }
   if (needhelp) {
      help();
      exit(1);
   }
   if (set)
      touch(filename);
   if (log)
      if (!set)
         printlog();
      else
         add2log();
   if (set)
      puts("...");
   if (!log && !set)
      writeout();
   fcloseall();
}           /* main */

void
touch(char name[])
{
   FILE           *file;
   char            tmp;

   int             modifiers;

   modifiers = bioskey(2);
   if (!(modifiers & RIGHT || modifiers & LEFT)) {
      printf("Updating launch time");
      file = fopen(name, "r+");
      fseek(file, SEEK_SET, 0);
      fread(&tmp, 1, 1, file);
      rewind(file);
      fwrite(&tmp, 1, 1, file);

   } else {
      printf("Aborting update of %s\n", _argv[0]);
      exit(1);
   }
}           /* touch */

char           *
upit(char str[])
{
   int             i;

   for (i = 0; i < strlen(str); i++) {
      str[i] = toupper(str[i]);
   }
   return str;
}


void
writeout(void)
{
   char tz[257],*oldtz;

   tz[0]='\0';
   oldtz=getenv("TZ");
   if(oldtz!=NULL){
     strcat(tz,"TZ=");
     putenv(tz);
   }

   findfirst(filename, &ffblk, FA_DIREC);
   old.tm_sec = 2 * (ffblk.ff_ftime & 31);
   old.tm_min = (ffblk.ff_ftime & 2016) >> 5;
   old.tm_hour = (ffblk.ff_ftime & 63488) >> 11;
   old.tm_mday = (ffblk.ff_fdate & 31);
   old.tm_mon = ((ffblk.ff_fdate & 480) >> 5) - 1;
   old.tm_year = ((ffblk.ff_fdate & 65024) >> 9) + 80;
   mktime(&old);

   thetime = time(NULL);
   new = localtime(&thetime);

   sec = (*new).tm_sec - old.tm_sec;
   min = (*new).tm_min - old.tm_min;
   if (sec < 0) {
      sec += 60;
      min--;
   }
   hour = (*new).tm_hour - old.tm_hour;
   if (min < 0) {
      min += 60;
      hour--;
   }
   day = (*new).tm_yday - old.tm_yday + (365 * ((*new).tm_year - old.tm_year));
   if (hour < 0) {
      hour += 24;
      day--;
   }
   sscanf(asctime(&old), "%3s %3s %d %d:%d:%d %d", bday, bmon, &bdate, &bhour, &bmin, &bsec, &byear);
   if (bhour >= 12) {
      bhour -= 12;
      ampm = 'p';
   }
   if (bhour == 0)
      bhour = 12;
   if (day < 0)
      printf("Error:  System clock has been changed since bootup");
   else if (day != 0)
      printf("System launched at %2.2d:%2.2d:%2.2d%cm on %s, %s %d, %d\nUp for %d day(s), %2.2d:%2.2d:%2.2d", bhour, bmin, bsec, ampm, bday, bmon, bdate, byear, day, hour, min, sec);
   else
      printf("System launched at %2.2d:%2.2d:%2.2d%cm on %s, %s %d, %d\nUp for %2.2d:%2.2d:%2.2d", bhour, bmin, bsec, ampm, bday, bmon, bdate, byear, hour, min, sec);

   if(oldtz!=NULL){
     strcpy(tz,oldtz);
     putenv(tz);
   }

}

void
help(void)
{
   printf("\nUptime %s                                      by Eric S. Peters\n",VER);
   printf("Compiled at %s, on %s\n\n", __TIME__, __DATE__);
   puts("Uptime will tell you how long your computer has been running since the");
   puts("last time it was launched.  Executed with no parameters, it returns the ");
   puts("time elapsed since bootup.\n");
   puts("SET\t: Will set the launchtime.  This needs to be in the AUTOEXEC.BAT file.");
   puts("\t: If either shift key is held, the set is aborted");
   puts("LOG\t: Will print the log, or add to the log if called with set");
   puts("\t: log will be in the same directory as uptime");
   puts("\n\nUptime can be copied and distributed freely.\n");
   puts("I really can't think of how a bug could get into a simple program like this");
   puts("but if one does pop up, let me know at peters@cs.colostate.edu");
   puts("or ericpete.comemsys@cms.gr.hp.com");
}

void
printlog()
{
   char            logname[255], line[81], ans, tail[10][81];
   FILE           *log;
   int             cnt = 0;

   getlog(logname);
   log = fopen(logname, "r");
   if (log != NULL) {
      while (fgets(line, 81, log) != NULL) {
         printf("%s", line);
	 if (++cnt % 24 == 0&&eof(log)!=0) {
            printf("(T)ail, (Q)uit, or anything else\r");
            ans = toupper(getch());
            switch (ans) {
	       case 'Q':
	       case 27 :
			 exit(0);
			 break;
	       case 'T':
			 cnt = 0;
			 clrscr();
			 fseek(log, 0, SEEK_SET);
			 puts("Last 10 Launches:\n");
			 while (fgets(tail[cnt++ % 10], 81, log) != NULL);
			 cnt--;
			 for (ans = 0; ans <= 9; ans++)
			    printf("%s", tail[cnt++ % 10]);
			 exit(0);
			 break;
	    }
	 }
      }
      printf("\nDelete the log?");
      ans = toupper(getch());
      if (ans != 'Y')
         ans = 'N';
      printf("%c", ans);
      if (ans == 'Y')
         unlink(logname);
   } else
      puts("No log to print");
}

void
getlog(char newname[])
{
   char           *letter;

   strcpy(newname, filename);
   letter = strrchr(newname, '.');
   strcpy(letter, ".LOG");

}

void
add2log()
{
   char            logname[255];
   FILE           *log;

   printf(" and adding entry to log");
   getlog(logname);
   log = fopen(logname, "at");
   time(&thetime);
   fprintf(log, "launched at: %s", ctime(&thetime));
}
